----------------------------------------
-- explode
----------------------------------------
on explode (delim, str, limit)
  l=[]
  if voidP(str) then return l
  if voidP(limit) then limit=the maxinteger
  dl=length(delim)
  
  repeat while true
    pos=offset(delim,str)
    if pos=0 then exit repeat
    
    s=str.char[1..pos-1]
    l.add(s)
    
    delete char 1 to pos+dl-1 of str
    if l.count=limit-1 then
      l.add(str)
      return l
    end if
  end repeat
  
  if pos=0 then pos=1-dl
  l.add(str.char[pos+dl..length(str)])
  
  return l
end

----------------------------------------
-- 
----------------------------------------
on implode (delim, l)
  str=""
  repeat with i=1 to l.count
    put l[i]&delim after str
  end repeat
  return str.char[1..(str.length-delim.length)]
end

----------------------------------------
-- 4 bytes
----------------------------------------
on dec2hex d
  v1 = d / 65536
  remainder = d mod 65536
  col = rgb(v1, (remainder / 256), (remainder mod 256))
  return (col.hexString().char[4..7])
end

----------------------------------------
-- returns binary string representation for number (little-endian)
----------------------------------------
on getBinStr (n, digits)
  s=""
  repeat with i = digits down to 1
    ex = power(256,i-1)
    put numtochar(floor(n/ex)) before s
    n = n mod ex
  end repeat
  return s
end

----------------------------------------
-- returns binary string interpreted as number (little-endian)
----------------------------------------
on getNum (binStr)
  n=0
  digits=binStr.length
  repeat with i = 1 to digits
    n=n+integer(power(256,i-1))*chartonum(binStr.char[i])
  end repeat
  return n
end

----------------------------------------
--
----------------------------------------
on floor n
  return bitOr( n, 0 )
end

----------------------------------------
-- http://en.wikipedia.org/wiki/INI_file
----------------------------------------
on iniToPropList (tFile)
  buffer = bx_file_get_contents(tFile)
  tList=[:]
  if stringP(buffer) then
    lc = buffer.line.count
    _player.itemDelimiter = "="
    
    tmp = tList
    prop = VOID
    
    repeat with i = 1 to lc
      tLine = buffer.line[i]
      if tLine starts numtochar(10) then delete char 1 of tLine
      if tLine starts ";" then next repeat -- comment
      if tLine starts "[" then
        prop = tLine.char[2..tLine.char.count-1]
        tList[prop] = [:]
        tmp = tList[prop]
        
      else if tLine.item.count>1 then
        prop = tLine.item[1]
        val = tLine.item[2]
        val = val.word[1..val.word.count] -- trim leading spaces
        tmp[prop] = val
      end if
    end repeat
  end if
  return tList
end

----------------------------------------
-- 
----------------------------------------
on getFileType (fn)
  od = _player.itemdelimiter
  _player.itemdelimiter = "."
  ftype = the last item of fn
  _player.itemdelimiter = od
  return ftype
end

----------------------------------------
-- 
----------------------------------------
on getFile (fn)
  od = _player.itemdelimiter
  pd = the last char of _movie.path
  _player.itemdelimiter = pd
  fn=the last item of fn
  _player.itemdelimiter = od
  return fn
end

----------------------------------------
-- 
----------------------------------------
on getFileName (fn)
  od = _player.itemdelimiter
  pd = the last char of _movie.path
  _player.itemdelimiter = pd
  fn=the last item of fn
  _player.itemdelimiter = "."
  if fn.item.count>1 then delete the last item of fn
  _player.itemdelimiter = od
  return fn
end

----------------------------------------
-- 
----------------------------------------
on getFolder (fn)
  od = _player.itemdelimiter
  pd = the last char of _movie.path
  _player.itemdelimiter = pd
  delete the last item of fn
  _player.itemdelimiter = od
  return fn&pd
end

----------------------------------------
-- was: get_files
----------------------------------------
on getFiles(folderPath, types)
  fl = []
  i=1
  repeat while true
    f = getNthFileNameInFolder(folderPath, i)
    if f=EMPTY then exit repeat
    if types.getPos(f.char[f.length-3..f.length]) then
    	fl.add(f)
    end if
    i=i+1
  end repeat
  return fl
end

----------------------------------------
-- finds largest rect with given ratio inside given rect
----------------------------------------
on findMaxRect (tMaxRect, tRatio)
  
  if tMaxRect.width=0 OR tMaxRect.height=0 OR tRatio=0 then
    return tMaxRect
  end if
  
  tMaxRectRatio = float(tMaxRect.width)/tMaxRect.height
  
  if tMaxRectRatio>=tRatio then
    h = tMaxRect.height
    w = h * tRatio
    x = (tMaxRect.width-w)/2
    y = 0
  else
    w = tMaxRect.width
    h = w / tRatio
    x = 0
    y = (tMaxRect.height-h)/2
  end if
  
  return rect(x, y, x+w, y+h)
end

----------------------------------------
--
----------------------------------------
--on checkXtra (tXtraName)
--  cnt = the number of xtras
--  repeat with i = 1 to cnt
--    put xtra(i).version
--    if xtra(i).name = tXtraName then return true
--  end repeat
--  return false
--end

on checkXtra (tXtraName, tMinVersion)
  repeat with x in _player.scriptingXtraList
    if string(x.name) = tXtraName then
      if voidP(tMinVersion) then
        return true
      else
        if x.version="" then return true -- ???
        --put x.version
        _player.itemdelimiter = "."
        cnt = min(tMinVersion.item.count, x.version.item.count)
        repeat with i = 1 to cnt
          v1 = integer(x.version.item[i])
          v2 = integer(tMinVersion.item[i])
          if v1 > v2 then return true
          else if v1 < v2 then return false
        end repeat
        return true
      end if
    end if
  end repeat
  return false
end

----------------------------------------
-- replace in string
----------------------------------------
on str_replace (stringToFind, stringToInsert, input)
  output = ""
  findLen = stringToFind.length - 1
  repeat while true
    currOffset = offset(stringToFind, input)
    if currOffset=0 then exit repeat
    put input.char [1..currOffset] after output
    delete the last char of output
    put stringToInsert after output
    delete input.char [1.. (currOffset + findLen)] 
  end repeat
  put input after output
  return output
end

----------------------------------------
-- returns commandline-params as list (without quotes)
----------------------------------------
on parseCommandLine
  cl=the commandLine
  l=[]
  if cl<>"" then
    tmp=cl
    cnt=length(cl)
    n=0
    repeat with i = 1 to cnt
      c=cl.char[i]
      if c=QUOTE then n=n+1
      if c=SPACE then
        if n mod 2 = 0 then 
          put "?" into char i of tmp
        end if
      end if
    end repeat
    the itemdelimiter="?"
    cnt=tmp.item.count
    repeat with i = 1 to cnt
      l.add(str_replace(QUOTE,"",tmp.item[i]))
    end repeat
  end if
  return l
end

----------------------------------------
-- 
----------------------------------------
on getBmpFromDib (str)
  
  bfSize=14 + str.length --size, in bytes, of the bitmap file.
  dibHeaderSize = getNum(str.char[1..4])
  biBitCount = getNum(str.char[15..16])
  if biBitCount=8 then -- palette
    bfOffBits = 54 + 1024
  else -- no palette
    bfOffBits = 54
  end if
  
  bmpHeader="BM"
  put getBinStr(bfSize+2, 4) after bmpHeader
  put numtochar(0)&numtochar(0)&numtochar(0)&numtochar(0) after bmpHeader
  put getBinStr(bfOffBits, 4) after bmpHeader
  
  put bmpHeader before str
  
  put numtochar(0)&numtochar(0) after str
  return str
end

----------------------------------------
-- 
----------------------------------------
on getDibFromBmp (data)
	delete char 1 to 14 of data
	delete char data.length-1 to data.length of data
  return data
end